#include "CellManager.h"

#include "EasyOsk.h"
#include "EasyMessage.h"

#include "GraphicsObject.h"

#include <pspdebug.h>
#include <pspctrl.h>
#include <pspgu.h>
#include <exception>
#include <vector>
#include <string>

CellManager::CellManager(void)
{
	mWidth=mHeight=100;
	initalise(1,1);
}

CellManager::~CellManager(void)
{
}

void CellManager::initalise(unsigned char width,unsigned char height)
{
	if(mWidth!=width || mHeight!=height)
	{
		mWidth=width;
		mHeight=height;

		short baseX=240-((mWidth*2)/2);
		short baseY=136-((mHeight*2)/2);

		try{
			delete[] mGrid;
		}catch(...){}

		mGrid=new Cell[mHeight*mWidth];

		for(int y=0;y<mHeight;y++)
		{
			for(int x=0;x<mWidth;x++)
			{
				mGrid[y*mWidth+x].setPosition(x*2+baseX,y*2+baseY);
				mGrid[y*mWidth+x].setDimention(2,2);
				mGrid[y*mWidth+x].setAlive(0);
				mGrid[y*mWidth+x].swap();
			}
		}
	}
}

unsigned char CellManager::getWidth()
{
	return mWidth;
}

unsigned char CellManager::getHeight()
{
	return mHeight;
}

void CellManager::setAlive(unsigned char x,unsigned char y)
{
	if(x>=0 && x<mWidth && y>=0 && y<mHeight)
	{
		mGrid[y*mWidth+x].setAlive(1);
		mGrid[y*mWidth+x].swap();
	}

}

void CellManager::setDeath(unsigned char x,unsigned char y)
{
	if(x>=0 && x<mWidth && y>=0 && y<mHeight)
	{
		mGrid[y*mWidth+x].setAlive(0);
		mGrid[y*mWidth+x].swap();
	}
}

void CellManager::update()
{
	unsigned char numb=0;
	for(int y=0;y<mHeight;y++)
	{
		for(int x=0;x<mWidth;x++)
		{
			numb=0;
			if(x==0 && y==0)
			{
				numb+=mGrid[(y+1)*mWidth+x+1].getAlive();
				numb+=mGrid[(y+1)*mWidth+x].getAlive();
				numb+=mGrid[y*mWidth+x+1].getAlive();
			}
			else if(x==(mWidth-1) && y==0)
			{
				numb+=mGrid[(y+1)*mWidth+x-1].getAlive();
				numb+=mGrid[(y+1)*mWidth+x].getAlive();
				numb+=mGrid[y*mWidth+x-1].getAlive();
			}
			else if(x==0 && y==(mHeight-1))
			{
				numb+=mGrid[(y-1)*mWidth+x+1].getAlive();
				numb+=mGrid[(y-1)*mWidth+x].getAlive();
				numb+=mGrid[y*mWidth+x+1].getAlive();
			}
			else if(x==(mWidth-1) && y==(mHeight-1))
			{
				numb+=mGrid[(y-1)*mWidth+x-1].getAlive();
				numb+=mGrid[(y-1)*mWidth+x].getAlive();
				numb+=mGrid[y*mWidth+x-1].getAlive();
			}
			else if(x==0)
			{
				numb+=mGrid[(y-1)*mWidth+x+1].getAlive();
				numb+=mGrid[y*mWidth+x+1].getAlive();
				numb+=mGrid[(y+1)*mWidth+x+1].getAlive();
				numb+=mGrid[(y-1)*mWidth+x].getAlive();
				numb+=mGrid[(y+1)*mWidth+x].getAlive();
			}
			else if(x==(mWidth-1))
			{
				numb+=mGrid[(y-1)*mWidth+x-1].getAlive();
				numb+=mGrid[y*mWidth+x-1].getAlive();
				numb+=mGrid[(y+1)*mWidth+x-1].getAlive();
				numb+=mGrid[(y-1)*mWidth+x].getAlive();
				numb+=mGrid[(y+1)*mWidth+x].getAlive();
			}
			else if(y==0)
			{
				numb+=mGrid[(y+1)*mWidth+x-1].getAlive();
				numb+=mGrid[(y+1)*mWidth+x].getAlive();
				numb+=mGrid[(y+1)*mWidth+x+1].getAlive();
				numb+=mGrid[y*mWidth+x-1].getAlive();
				numb+=mGrid[y*mWidth+x+1].getAlive();
			}
			else if(y==(mHeight-1))
			{
				numb+=mGrid[(y-1)*mWidth+x-1].getAlive();
				numb+=mGrid[(y-1)*mWidth+x].getAlive();
				numb+=mGrid[(y-1)*mWidth+x+1].getAlive();
				numb+=mGrid[y*mWidth+x-1].getAlive();
				numb+=mGrid[y*mWidth+x+1].getAlive();
			}
			else
			{
				numb+=mGrid[(y-1)*mWidth+x].getAlive();
				numb+=mGrid[(y+1)*mWidth+x].getAlive();
				numb+=mGrid[(y-1)*mWidth+x-1].getAlive();
				numb+=mGrid[(y+1)*mWidth+x-1].getAlive();
				numb+=mGrid[y*mWidth+x-1].getAlive();
				numb+=mGrid[(y-1)*mWidth+x+1].getAlive();
				numb+=mGrid[(y+1)*mWidth+x+1].getAlive();
				numb+=mGrid[y*mWidth+x+1].getAlive();
			}

			if (mGrid[y*mWidth+x].getAlive())
			{
				switch(numb)
				{
				case 0:
				case 1:
				case 4:
				case 5:
				case 6:
				case 7:
				case 8:
					mGrid[y*mWidth+x].setAlive(0);
					break;
				default:
					break;
				}
			}
			else 
			{
				switch(numb)
				{
				case 3:
					mGrid[y*mWidth+x].setAlive(1);
					break;
				default:
					break;
				}
			}
		}
	}

	for(int x=0;x<mWidth;x++)
	{
		for(int y=0;y<mHeight;y++)
		{			
			mGrid[y*mWidth+x].swap();
		}
	}
}

void CellManager::clear()
{
	for(int x=0;x<mWidth;x++)
	{
		for(int y=0;y<mHeight;y++)
		{
			mGrid[y*mWidth+x].setAlive(0);
			mGrid[y*mWidth+x].swap();
		}
	}
}

void CellManager::render()
{
	for(int x=0;x<mWidth;x++)
	{
		for(int y=0;y<mHeight;y++)
		{
			mGrid[y*mWidth+x].render();
		}
	}
}

void CellManager::saveState()
{
	for(int x=0;x<mWidth;x++)
	{
		for(int y=0;y<mHeight;y++)
		{
			mSave[y][x]=mGrid[y*mWidth+x].getAlive();
		}
	}
}

void CellManager::loadState()
{
	for(int x=0;x<mWidth;x++)
	{
		for(int y=0;y<mHeight;y++)
		{
			mGrid[y*mWidth+x].setAlive(mSave[y][x]);
			mGrid[y*mWidth+x].swap();
		}
	}
}

void CellManager::saveFile()
{
	EasyOsk osk;
	EasyMessage msg;

	char file[32];

	osk.SetDescription("Name"); 
	osk.GetString(file,32,"");

	std::string filename;
	filename+=file;
	filename+=".life";


	SceUID fd=sceIoOpen(filename.c_str(),PSP_O_CREAT | PSP_O_WRONLY,0777);
	if(fd)
	{
		unsigned char tSize[2];
		tSize[0]=mWidth;
		tSize[1]=mHeight;
		sceIoWrite(fd,tSize,2);

		char death='0';
		char alive='1';

		for(int x=0;x<mWidth;x++)
		{
			for(int y=0;y<mHeight;y++)
			{
				if(mGrid[y*mWidth+x].getAlive())
					sceIoWrite(fd,&alive,1);
				else
					sceIoWrite(fd,&death,1);
			}
		}
		sceIoClose(fd);
		std::string message;
		message+="File saved as:";
		message+=file;
		message+=".life";

		msg.ShowMessage(message.c_str(),0);
	}
	else
	{
		msg.ShowMessage("File not saved.",0);
	}
}

void CellManager::loadFile()
{
	EasyMessage msg;
	SceUID dir;
	dir = sceIoDopen(".");
	if(dir)
	{
		std::vector<SceIoDirent> filelist;
		SceIoDirent entry;

		while(sceIoDread(dir,&entry))
		{
			if(strcmp(entry.d_name,"EBOOT.PBP")==0 || strcmp(entry.d_name,".")==0 || strcmp(entry.d_name,"..")==0)
			{				
			}
			else
			{
				filelist.push_back(entry);
			}
		}
		sceIoDclose(dir);
		if(filelist.size()>0)
		{
			unsigned char count=0;
			unsigned char cursor=0;
			SceCtrlData pad;
			msg.ShowMessage("testpoint -1",0);
			bool go=1;
			while(go)
			{
				GraphicsObject::endFrame();
				GraphicsObject::startFrame();

				pspDebugScreenSetXY(0,0);
				pspDebugScreenPrintf("Files:\n");
				count=0;
				for(std::vector<SceIoDirent>::size_type i=0;i<filelist.size() && count <= 32;i++)
				{
					if(i==cursor)
						pspDebugScreenPrintf("->%s\n",filelist[i].d_name);
					else
						pspDebugScreenPrintf("  %s\n",filelist[i].d_name);
					count++;
				}
				if(cursor>count)
					cursor=count-2;

				sceCtrlReadBufferPositive(&pad,1);
				if(pad.Buttons & PSP_CTRL_UP && cursor>0)
					cursor--;
				if(pad.Buttons & PSP_CTRL_DOWN && cursor<32)
					cursor++;

				if(pad.Buttons & PSP_CTRL_CROSS){
					msg.ShowMessage("testpoint 0",0);
					go=0;
				}
			}
			msg.ShowMessage("testpoint 1",0);
			SceUID fd = sceIoOpen(filelist[cursor].d_name,PSP_O_RDONLY,0777);
			if(fd)
			{
				unsigned char tSize[2];
				sceIoRead(fd,tSize,2);
				msg.ShowMessage("testpoint 2",0);
				if(tSize[0]>=20 && tSize[0]<=240 && tSize[1]>=20 && tSize[1]<=136)
				{
					msg.ShowMessage("testpoint 3",0);
					char tGrid[tSize[0]*tSize[1]];
					int tBytes=sceIoRead(fd,tGrid,tSize[0]*tSize[1]);
					if(tBytes==tSize[0]*tSize[1])
					{
						initalise(tSize[0],tSize[1]);
						clear();
						for(int x=0;x<mWidth;x++)
						{
							for(int y=0;y<mHeight;y++)
							{
								if(tGrid[y*mWidth+x]=='1')
									setAlive(x,y);
								else
									setDeath(x,y);
							}
						}
						msg.ShowMessage("testpoint 4",0);
					}
					else
					{
						msg.ShowMessage("File corrupted",0);
					}
				}
				else
				{
					msg.ShowMessage("File corrupted",0);
				}
				sceIoClose(fd);
			}
			else
			{
				msg.ShowMessage("Error opening file",0);
			}
		}
		else
		{
			msg.ShowMessage("No files found",0);
		}
	}
	else
	{
		msg.ShowMessage("Error while searching for files.",0);
	}
}

