 /*  
  * File:   easyosk.cpp 
  * Author: joris 
  *  
  * Created on 21 augustus 2009, 13:07 
  */ 
  
 #include "easyosk.h" 
  
 static unsigned int __attribute__((aligned(16))) list[262144]; 
  
 EasyOsk::EasyOsk() { 
     memset(desc,0,sizeof(desc)); 
     memset(&data, 0, sizeof (SceUtilityOskData)); 
     data.language = PSP_UTILITY_OSK_LANGUAGE_DEFAULT; // Use system default for text input 
     data.lines = 1; 
     data.unk_24 = 1; 
     data.inputtype = PSP_UTILITY_OSK_INPUTTYPE_ALL; // Allow all input types 
     data.desc = desc; 
     data.intext = NULL; 
     data.outtextlength = 0; 
     data.outtextlimit = 0; 
     data.outtext = NULL; 
     memset(&params, 0, sizeof (SceUtilityOskParams)); 
     params.base.size = sizeof (SceUtilityOskParams); 
     sceUtilityGetSystemParamInt(PSP_SYSTEMPARAM_ID_INT_LANGUAGE, &params.base.language); 
     sceUtilityGetSystemParamInt(PSP_SYSTEMPARAM_ID_INT_UNKNOWN, &params.base.buttonSwap); 
     params.base.graphicsThread = 17; 
     params.base.accessThread = 19; 
     params.base.fontThread = 18; 
     params.base.soundThread = 16; 
     params.datacount = 1; 
     params.data = &data; 
 } 
  
 EasyOsk::~EasyOsk() { 
 } 
  
 void EasyOsk::InitGU() { 
     sceGuInit(); 
     sceGuStart(GU_DIRECT, list); 
     sceGuDrawBuffer(GU_PSM_8888, (void*) 0, 512); 
     sceGuDispBuffer(480, 272, (void*) 0x88000, 512); 
     sceGuDepthBuffer((void*) 0x110000, 512); 
     sceGuOffset(2048 - (480 / 2), 2048 - (272 / 2)); 
     sceGuViewport(2048, 2048, 480, 272); 
     sceGuDepthRange(0xc350, 0x2710); 
     sceGuScissor(0, 0, 480, 272); 
     sceGuEnable(GU_SCISSOR_TEST); 
     sceGuDepthFunc(GU_GEQUAL); 
     sceGuEnable(GU_DEPTH_TEST); 
     sceGuFrontFace(GU_CW); 
     sceGuShadeModel(GU_SMOOTH); 
     sceGuEnable(GU_CULL_FACE); 
     sceGuEnable(GU_CLIP_PLANES); 
     sceGuEnable(GU_BLEND); 
     sceGuBlendFunc(GU_ADD, GU_SRC_ALPHA, GU_ONE_MINUS_SRC_ALPHA, 0, 0); 
     sceGuFinish(); 
     sceGuSync(0, 0); 
  
     sceDisplayWaitVblankStart(); 
     sceGuDisplay(GU_TRUE); 
 } 
  
 void EasyOsk::TermGU() { 
     sceGuTerm(); 
 } 
  
 int EasyOsk::GetString(char * buf, unsigned int size, const char* initial) { 
  
     unsigned short tmpbuf[size]; 
     memset(tmpbuf,0,size); 
  
     unsigned short tmpinit[size]; 
     memset(tmpinit,0,size); 
     for(unsigned int i=0;i<size;i++){ 
         tmpinit[i]=initial[i]; 
     } 
  
     data.outtext = tmpbuf; 
     data.outtextlength = size; 
     data.outtextlimit = size; 
     data.intext = tmpinit; 
  
     sceUtilityOskInitStart(&params); 
  
     int done = 0; 
     while (!done) { 
         sceGuStart(GU_DIRECT, list); 
         sceGuClearColor(0); 
         sceGuClearDepth(0); 
         sceGuClear(GU_COLOR_BUFFER_BIT | GU_DEPTH_BUFFER_BIT); 
         sceGuFinish(); 
         sceGuSync(0, 0); 
  
         switch (sceUtilityOskGetStatus()) { 
             case PSP_UTILITY_DIALOG_INIT: 
                 break; 
  
             case PSP_UTILITY_DIALOG_VISIBLE: 
                 sceUtilityOskUpdate(1); 
                 break; 
  
             case PSP_UTILITY_DIALOG_QUIT: 
                 sceUtilityOskShutdownStart(); 
                 break; 
  
             case PSP_UTILITY_DIALOG_FINISHED: 
                 break; 
  
             case PSP_UTILITY_DIALOG_NONE: 
                 done = 1; 
  
             default: 
                 break; 
  } 
  
         sceDisplayWaitVblankStart(); 
         sceGuSwapBuffers(); 
     } 
  
     for(unsigned int i=0;i<size;i++){ 
         buf[i]=(char)tmpbuf[i]; 
     } 
  
     return params.base.result; 
 } 
  
 void EasyOsk::SetDescription(const char* description){ 
     memset(desc,0,sizeof(desc)); 
     for(unsigned int i=0;i<128;i++){ 
         desc[i]=description[i]; 
     } 
     data.desc=desc; 
 } 
  
 void EasyOsk::SetLines(int lines){ 
     data.lines=lines; 
 } 
 