#include "GameApp.h"

#include <pspgu.h>
#include <psppower.h>

GameApp::GameApp()
{
	mRunning=0;
	mLoaded=0;
	mPauzed=1;
	mState=PspLifeStateStart;
	mMenuItem=0;
	mMenuWidth=mMenuHeight=100;
	mCpuSpeed=333;
}

GameApp::~GameApp()
{
}

void GameApp::initalise()
{

	scePowerSetClockFrequency(333,333,166);
	intraFontInit();
	mGfx.initGraphics();
	mFont=intraFontLoad("flash0:/font/ltn8.pgf",0);
	intraFontSetStyle(mFont, 1.0f, 0xFFFFFFFF, 0xFF3F3F3F, 0);	
	mCellMgr.initalise(100,100);
	mRunning=1;
	mLoaded=1;
}

void GameApp::update()
{
	sceCtrlSetSamplingMode(1);
	sceCtrlPeekBufferPositive(&pad,1);

	switch(mState){
		case PspLifeStateStart:	
			mState=PspLifeStateMenu;
			break;

		case PspLifeStateMenu:
			if(pad.Buttons & PSP_CTRL_LEFT)
			{
				if(mMenuItem==0)
				{					
					if (mMenuWidth>20) mMenuWidth-=1;
				}
				else if(mMenuItem==1)
				{					
					if (mMenuHeight>20) mMenuHeight-=1;
				}
			}

			if(pad.Buttons & PSP_CTRL_RIGHT)
			{
				if(mMenuItem==0)
				{
					mMenuWidth+=1;
					if (mMenuWidth>240) mMenuWidth=240;
				}
				else if(mMenuItem==1)
				{
					mMenuHeight+=1;
					if (mMenuHeight>136) mMenuHeight=136;
				}
			}

			if(pad.Buttons & PSP_CTRL_LTRIGGER)
			{
				switch(mCpuSpeed)
				{
				case 333:
					mCpuSpeed=266;
					break;
				case 266:
					mCpuSpeed=222;
					break;
				case 222:
					mCpuSpeed=166;
					break;
				case 166:
					mCpuSpeed=111;
					break;
				}
				scePowerSetClockFrequency(mCpuSpeed,mCpuSpeed,mCpuSpeed/2);
				while(pad.Buttons & PSP_CTRL_LTRIGGER)
					sceCtrlPeekBufferPositive(&pad,1);
			}

			if(pad.Buttons & PSP_CTRL_RTRIGGER)
			{
				switch(mCpuSpeed)
				{
				case 111:
					mCpuSpeed=166;
					break;
				case 266:
					mCpuSpeed=333;
					break;
				case 222:
					mCpuSpeed=266;
					break;
				case 166:
					mCpuSpeed=222;
					break;
				}
				scePowerSetClockFrequency(mCpuSpeed,mCpuSpeed,mCpuSpeed/2);
				while(pad.Buttons & PSP_CTRL_RTRIGGER)
					sceCtrlPeekBufferPositive(&pad,1);
			}


			if(pad.Buttons & PSP_CTRL_UP)
			{
				mMenuItem-=1;
				if(mMenuItem<0)mMenuItem=0;
				while(pad.Buttons & PSP_CTRL_UP)
					sceCtrlPeekBufferPositive(&pad,1);
			}
			else if(pad.Buttons & PSP_CTRL_DOWN)
			{
				mMenuItem+=1;
				if(mMenuItem>6)mMenuItem=6;
				while(pad.Buttons & PSP_CTRL_DOWN)
					sceCtrlPeekBufferPositive(&pad,1);
			}

			else if(pad.Buttons & PSP_CTRL_CROSS)
			{
				switch(mMenuItem)
				{
				case 0:
				case 1:
					break;
				case 2:
					mState=PspLifeStateGame;
					mCellMgr.initalise(mMenuWidth,mMenuHeight);
					mCursor.setDimention(mMenuWidth,mMenuHeight);
					mPauzed=1;
					break;
				case 3:
					mCellMgr.saveFile();
					break;
				case 4:
					mCellMgr.loadFile();
					break;
				case 5:
					mState=PspLifeStateHelp;
					break;
				case 6:
					mState=PspLifeStateCredits;
					break;
				}
				while(pad.Buttons & PSP_CTRL_CROSS)
					sceCtrlPeekBufferPositive(&pad,1);
			}
			break;

		case PspLifeStateGame:
			if(pad.Buttons & PSP_CTRL_LEFT || pad.Lx < 48)
			{
				mCursor.translate(-1,0);
				while(pad.Buttons & PSP_CTRL_LEFT)
					sceCtrlPeekBufferPositive(&pad,1);	
			}
			if(pad.Buttons & PSP_CTRL_RIGHT || pad.Lx > 208)
			{
				mCursor.translate(1,0);
				while(pad.Buttons & PSP_CTRL_RIGHT)
					sceCtrlPeekBufferPositive(&pad,1);	
			}
			if(pad.Buttons & PSP_CTRL_UP || pad.Ly < 48)
			{
				mCursor.translate(0,-1);
				while(pad.Buttons & PSP_CTRL_UP)
					sceCtrlPeekBufferPositive(&pad,1);	
			}
			if(pad.Buttons & PSP_CTRL_DOWN || pad.Ly > 208)
			{
				mCursor.translate(0,1);
				while(pad.Buttons & PSP_CTRL_DOWN)
					sceCtrlPeekBufferPositive(&pad,1);	
			}

			if (mPauzed)
			{
				if(pad.Buttons & PSP_CTRL_CROSS)
					mCellMgr.setAlive(mCursor.getPosX(),mCursor.getPosY());
				if(pad.Buttons & PSP_CTRL_CIRCLE)
					mCellMgr.setDeath(mCursor.getPosX(),mCursor.getPosY());

				if(pad.Buttons & PSP_CTRL_TRIANGLE)
					mCellMgr.clear();
			}

			if(pad.Buttons & PSP_CTRL_START)
			{
				mPauzed=!mPauzed;
				while(pad.Buttons & PSP_CTRL_START)
					sceCtrlPeekBufferPositive(&pad,1);				
			}	
			else if (!mPauzed)
			{
				mCellMgr.update();
			}	

			if(pad.Buttons & PSP_CTRL_SELECT)
			{
				mState=PspLifeStateMenu;			
			}	

			if(pad.Buttons & PSP_CTRL_LTRIGGER)
			{
				mCellMgr.loadState();
			}

			if(pad.Buttons & PSP_CTRL_RTRIGGER)
			{
				mCellMgr.saveState();
			}

			break;

		case PspLifeStateHelp:
			if(pad.Buttons & PSP_CTRL_START)
			{
				while(pad.Buttons & PSP_CTRL_START)
					sceCtrlPeekBufferPositive(&pad,1);
				mState=PspLifeStateMenu;
			}
			break;

		case PspLifeStateCredits:
			if(pad.Buttons & PSP_CTRL_START)
			{
				while(pad.Buttons & PSP_CTRL_START)
					sceCtrlPeekBufferPositive(&pad,1);
				mState=PspLifeStateMenu;
			}
			break;
	}
}

void GameApp::render()
{
	mGfx.startFrame();

	switch(mState){
		case PspLifeStateStart:			
			break;

		case PspLifeStateMenu:
			intraFontPrint(mFont,  2, 15,"PSPLife V0.2");
			intraFontPrint(mFont,  100, 50, "Grid Width:"); 
			intraFontPrint(mFont,  100, 80, "Grid Height:");

			if(mMenuItem==0)
				intraFontPrintf(mFont,  180, 50, ">%i<",mMenuWidth); 
			else
				intraFontPrintf(mFont,  180, 50, " %i",mMenuWidth); 

			if(mMenuItem==1)
				intraFontPrintf(mFont,  180, 80, ">%i<",mMenuHeight); 
			else
				intraFontPrintf(mFont,  180, 80, " %i",mMenuHeight); 

			if(mMenuItem==2)
				intraFontPrintf(mFont,  180, 110, ">Start<",mMenuWidth); 
			else
				intraFontPrintf(mFont,  180, 110, " Start",mMenuWidth); 

			if(mMenuItem==3)
				intraFontPrintf(mFont,  180, 140, ">Save< not complete yet",mMenuWidth); 
			else
				intraFontPrintf(mFont,  180, 140, " Save",mMenuWidth); 

			if(mMenuItem==4)
				intraFontPrintf(mFont,  180, 170, ">Load< not complete yet",mMenuWidth); 
			else
				intraFontPrintf(mFont,  180, 170, " Load",mMenuWidth); 

			if(mMenuItem==5)
				intraFontPrintf(mFont,  180, 200, ">Help<",mMenuWidth); 
			else
				intraFontPrintf(mFont,  180, 200, " Help",mMenuWidth); 

			if(mMenuItem==6)
				intraFontPrintf(mFont,  180, 230, ">Credits<",mMenuWidth); 
			else
				intraFontPrintf(mFont,  180, 230, " Credits",mMenuWidth); 

			intraFontPrintf(mFont,  100, 260, "CPU Speed: %i MHz",mCpuSpeed);

			break;

		case PspLifeStateGame:
			mCellMgr.render();
			if (mPauzed)
				mCursor.render();
			break;

		case PspLifeStateHelp:
			intraFontPrint(mFont,  2, 15, 
				"PSPLife V0.2\n"
				"------------\n"
				"Controlls:\n\n"
				"Analog stick/D-pad: Move cursor\n"
				"Cross: place cell\n"
				"Circle: remove cell\n"
				"Triangle: Clear field\n"
				"Start: Start/Stop simulation\n"
				"Select: Go back to menu\n\n"
				"L-Trigger: Load state\n"
				"R-Trigger: Save state\n\n"
				"To change CPU speed use the L- and R-Trigger in the menu."
				);
			break;

		case PspLifeStateCredits:
			intraFontPrint(mFont,  2, 15,
				"PSPLife V0.2\n"
				"------------\n"
				"Made by JoJoJoris\n"
				"jojojoris@hotmail.com\n\n"
				"intraFont by BenHur\n"
				"XMB Icon by cmbeke"
				);
			break;
	}

	mGfx.endFrame();
}

void GameApp::stop()
{
	mRunning=0;
}

void GameApp::unload()
{
	if(mLoaded)
	{
		mLoaded=0;
	}
}

bool GameApp::running()
{
	return mRunning;
}
